"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from matplotlib.figure import Figure
from xsugar import Experiment, ureg
from sugarplot import assert_figures_equal, prettifyPlot
from ast import literal_eval

def testSavePSDFigureFilename(exp, path_data, convert_name):
    """
    Tests that we successfully created and saved a single PSD figure when
    our dataset is just a single item.
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    condition_name = convert_name('TEST1~wavelengths-1~temperatures-25')
    filename_desired = condition_name + '~PSD.png'
    exp.data = {condition_name: raw_data}
    exp.plotPSD(average_along=None)
    file_found = os.path.isfile(path_data['figures_full_path'] + filename_desired)
    assert_equal(file_found, True)

def testSavePSDFigureMultipleFilename(exp, path_data, convert_name):
    """
    Tests that we successfully created and saved a single PSD figure when
    our dataset is just a single item.
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    condition_name_1 = convert_name('TEST1~wavelengths-1~temperatures-25~replicate-0')
    condition_name_2 = convert_name('TEST1~wavelengths-1~temperatures-25~replicate-1')
    filename_desired_1 = condition_name_1 + '~PSD.png'
    filename_desired_2 = condition_name_2 + '~PSD.png'
    exp.data = {condition_name_1: raw_data, condition_name_2: raw_data}
    exp.plotPSD(average_along=None)
    file1_found = os.path.isfile(path_data['figures_full_path'] +
                                 filename_desired_1)
    file2_found = os.path.isfile(path_data['figures_full_path'] +
                                 filename_desired_2)
    assert_equal(file1_found, True)
    assert_equal(file2_found, True)

def testSavePSDFigureAverageFilename(exp, path_data, convert_name):
    """
    Tests that we successfully created and saved a single PSD figure when
    we want to create an averaged PSD plot
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    raw_data_2 = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [8,4.5,8]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    condition_name_1 = convert_name('TEST1~wavelengths-1~temperatures-25~replicate-1')
    condition_name_2 = convert_name('TEST1~wavelengths-1~temperatures-25~replicate-2')
    filename_desired = convert_name('TEST1~wavelengths-1~temperatures-25~PSD~averaged.png')

    exp.data = {condition_name_1: raw_data,
                 condition_name_2: raw_data_2}
    exp.plotPSD(average_along='replicate')
    file_found = os.path.isfile(path_data['figures_full_path'] + filename_desired)
    assert_equal(file_found, True)

def testGenerateTimeDomainPlot(exp, path_data, convert_name):
    """
    Tests that we successfully create a simple figure from a single pandas
    array.
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    condition_name = convert_name('TEST1~wavelengths-1~temperatures-25_replicate-1')
    filename_desired = condition_name + '.png'
    exp.data = {condition_name: raw_data}
    exp.plot()
    file_found = os.path.isfile(path_data['figures_full_path'] + filename_desired)
    assert_equal(file_found, True)

def testGenerateRepresentativePlot(exp, path_data, convert_name):
    """
    Tests that we successfully create a single figure from a whole set of
    replicate data, instead of a bunch of figures
    """
    raw_data = pd.DataFrame({'Time (ms)': [1, 2, 3],
                             'Current (mV)': [4,4.5,6]})
    cond_1 = {'wavelength': 1, 'temperature': 25, 'frequency': 8500,
              'replicate': 1}
    cond_2 = {'wavelength': 1, 'temperature': 25, 'frequency': 8500,
              'replicate': 2}
    condition_name_1 = convert_name('TEST1~wavelengths-1~temperatures-25~replicate-1')
    condition_name_2 = convert_name('TEST1~wavelengths-1~temperatures-25~replicate-2')

    filename_desired = convert_name('TEST1~wavelengths-1~temperatures-25~representative')
    exp.data = {condition_name_1: raw_data,
                 condition_name_2 : raw_data}
    exp.plot(representative='replicate')
    files_found = os.listdir(path_data['figures_full_path'])
    file_1_found = \
        os.path.isfile(path_data['figures_full_path'] + filename_desired + '.png')
    assert_equal(file_1_found, True)
    assert_equal(len(files_found), 1)

def test_generate_plot_1var(exp, convert_name):
    names = [convert_name(name) for name in \
        [
        'TEST1~wavelength=1',
        'TEST1~wavelength=2',
        ]]
    data_dict = {
        names[0]: 1.0,
        names[1]: 2.0,
    }
    actual_figs, actual_axes = exp.plot(data_dict)
    actual_fig = actual_figs[0]
    actual_ax = actual_axes[0]
    desired_fig = Figure()
    desired_ax = desired_fig.subplots(subplot_kw={'xlabel': 'wavelength', 'ylabel': 'Value'})
    desired_ax.plot([1, 2], [1.0, 2.0])
    prettifyPlot(fig=desired_fig, ax=desired_ax)
    assert_figures_equal(actual_fig, desired_fig)

def test_generate_plot_1var_units(exp_units, convert_name):
    names = [convert_name(name) for name in \
        [
        'TEST1~wavelength=1nm',
        'TEST1~wavelength=2nm',
        ]]
    data_dict = {
        names[0]: 1.0 * ureg.nA,
        names[1]: 2.0 * ureg.nA,
    }
    actual_figs, actual_axes = exp_units.plot(data_dict)
    actual_fig = actual_figs[0]
    actual_ax = actual_axes[0]
    desired_fig = Figure()
    desired_ax = desired_fig.subplots(subplot_kw={'xlabel': 'wavelength (nm)', 'ylabel': 'current (nA)'})
    desired_ax.plot([1, 2], [1.0, 2.0])
    prettifyPlot(fig=desired_fig, ax=desired_ax)
    assert_figures_equal(actual_fig, desired_fig)

def test_generate_plot_2var(exp, convert_name):
    names = [convert_name(name) for name in \
        [
        'TEST1~wavelength=1~temperature=25.0',
        'TEST1~wavelength=2~temperature=25.0',
        'TEST1~wavelength=1~temperature=35.0',
        'TEST1~wavelength=2~temperature=35.0',
        ]]
    data_dict = {
        names[0]: 1.0,
        names[1]: 2.0,
        names[2]: 3.0,
        names[3]: 4.0,
    }
    actual_figs, actual_axes = exp.plot(data_dict)
    assert_equal(len(actual_figs), 2)
    assert_equal(len(actual_axes), 2)

    desired_fig0 = Figure()
    desired_ax0 = desired_fig0.subplots(subplot_kw={'xlabel': 'wavelength', 'ylabel': 'Value'})
    desired_ax0.plot([1, 2], [1.0, 2.0])
    desired_ax0.plot([1, 2], [3.0, 4.0])
    prettifyPlot(fig=desired_fig0, ax=desired_ax0)

    desired_fig1 = Figure()
    desired_ax1 = desired_fig1.subplots(subplot_kw={'xlabel': 'temperature', 'ylabel': 'Value'})
    desired_ax1.plot([25.0, 35.0], [1.0, 3.0])
    desired_ax1.plot([25.0, 35.0], [2.0, 4.0])
    prettifyPlot(fig=desired_fig1, ax=desired_ax1)

    assert_figures_equal(actual_figs[0], desired_fig0)
    assert_figures_equal(actual_figs[1], desired_fig1)



import pytest
import numpy as np
import pandas as pd
from pandas.testing import assert_frame_equal
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment, ureg
from ast import literal_eval
from itertools import zip_longest
from spectralpy import power_spectrum
from sciparse import assertDataDictEqual

@pytest.fixture
def exp(path_data):
    wavelengths = np.array([1, 2, 3])
    temperatures = np.array([25, 50])
    frequency = 8500
    exp = Experiment(name='TEST1', kind='test',
                     frequency=frequency,
                     wavelengths=wavelengths,
                     temperatures=temperatures)
    yield exp
    rmtree(path_data['data_base_path'], ignore_errors=True)
    rmtree(path_data['figures_base_path'], ignore_errors=True)
    rmtree(path_data['designs_base_path'], ignore_errors=True)


def testGenerateMasterData1Var(exp, exp_data):
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    name1 = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
        'temperature' + ns + '25'
    name2 = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
        'temperature' + ns + '35'
    scalar_data = {
        name1: 1,
        name2: 2,
    }
    desired_data = pd.DataFrame({
        'temperature': [25, 35],
        'Value': [1, 2]})
    actual_data = exp.master_data(data_dict=scalar_data)
    assert_frame_equal(actual_data, desired_data)

def test_master_data_units(exp_units, convert_name):
    name = convert_name('TEST1~wavelength=25nm~temperature=305K')
    data_dict = {name: ureg.mV * 1.5}
    desired_master_data = pd.DataFrame({
            'wavelength (nm)': [25],
            'temperature (K)': [305],
            'voltage (mV)': [1.5]})
    actual_master_data = exp_units.master_data(data_dict)
    assert_frame_equal(actual_master_data, desired_master_data)

def test_master_data_nodrop(exp_units, convert_name):
    """
    Checks that we do not drop the last column.
    """
    name1 = convert_name('TEST1~wavelength=25nm~temperature=305K')
    name2 = convert_name('TEST1~wavelength=35nm~temperature=306K')
    data_dict = {
        name1: ureg.mV * 1.5,
        name2: ureg.mV * 1.5,
    }
    desired_master_data = pd.DataFrame({
            'wavelength (nm)': [25, 35],
            'temperature (K)': [305, 306],
            'voltage (mV)': [1.5, 1.5]})
    actual_master_data = exp_units.master_data(data_dict)
    assert_frame_equal(actual_master_data, desired_master_data)

def testGenerateMasterData2Var(exp, exp_data):
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    name1 = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
        'temperature' + ns + '25'
    name2 = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
        'temperature' + ns + '35'
    name3 = 'TEST1' + js + 'wavelength' + ns + '2' + js + \
        'temperature' + ns + '25'
    name4 = 'TEST1' + js + 'wavelength' + ns + '2' + js + \
        'temperature' + ns + '35'
    scalar_data = {
               name1: 1,
               name2: 2,
               name3: 3,
               name4: 4}

    desired_data = pd.DataFrame({
        'wavelength': [1, 1, 2, 2],
        'temperature': [25, 35, 25, 35],
        'Value': [1, 2, 3, 4]})
    actual_data = exp.master_data(data_dict=scalar_data)
    assert_frame_equal(actual_data, desired_data)

def test_data_from_master(exp, exp_data):
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    master_data = pd.DataFrame({
        'wavelength': [1, 2],
        'Value': [3, 4]})
    name1 = 'TEST1' + js + 'wavelength' + ns + '1'
    name2 = 'TEST1' + js + 'wavelength' + ns + '2'
    desired_data = {
        name1: 3,
        name2: 4
    }
    actual_data = exp.data_from_master(master_data)
    assertDataDictEqual(actual_data, desired_data)

def test_data_from_master_units(exp_units, convert_name):
    desired_name = convert_name('TEST1~wavelength=25nm~temperature=305K')
    master_data = pd.DataFrame({
            'wavelength (nm)': [25],
            'temperature (K)': [305],
            'voltage (mV)': [1.5]})
    desired_data = {desired_name: 1.5 * ureg.mV}
    actual_data = exp_units.data_from_master(master_data)
    assertDataDictEqual(actual_data, desired_data)

def test_data_from_master_2var(exp, exp_data):
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    master_data = pd.DataFrame({
        'wavelength': [0, 1, 2, 0, 1, 2],
        'temperature': [25.0, 25.0, 25.0, 35.0, 35.0, 35.0],
        'Value': [0, 1, 2, 3, 4, 5]})
    names = [
        'TEST1' + js + 'wavelength' + ns + '0' + js + \
            'temperature' + ns + '25.0',
        'TEST1' + js + 'wavelength' + ns + '1' + js + \
            'temperature' + ns + '25.0',
        'TEST1' + js + 'wavelength' + ns + '2' + js + \
            'temperature' + ns + '25.0',
        'TEST1' + js + 'wavelength' + ns + '0' + js + \
            'temperature' + ns + '35.0',
        'TEST1' + js + 'wavelength' + ns + '1' + js + \
            'temperature' + ns + '35.0',
        'TEST1' + js + 'wavelength' + ns + '2' + js + \
            'temperature' + ns + '35.0',
    ]
    desired_data_dict = {
        names[0]: 0,
        names[1]: 1,
        names[2]: 2,
        names[3]: 3,
        names[4]: 4,
        names[5]: 5}
    actual_data_dict = exp.data_from_master(master_data)
    assertDataDictEqual(actual_data_dict, desired_data_dict)

def testGenerateMasterDataDict1Var(exp, exp_data):
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    name1 = 'TEST1' + js + 'wavelength' + ns + '1'
    name2 = 'TEST1' + js + 'wavelength' + ns + '2'
    name_all = 'TEST1' + js + 'wavelength' + ns + 'all'
    data_dict = {
        name1: 3.0,
        name2: 4.0}
    desired_data = {
        name_all: pd.DataFrame({
                'wavelength': [1, 2],
                'Value': [3.0, 4.0]})}
    actual_data = exp.master_data_dict(data_dict)
    assertDataDictEqual(actual_data, desired_data)

def test_master_data_dict_1var_units(exp_units, convert_name):
    name1 = convert_name('TEST1~wavelength=1nm')
    name2 = convert_name('TEST1~wavelength=2nm')
    name_all = convert_name('TEST1~wavelength=all')
    data_dict = {
        name1: 3.0 * ureg.nA,
        name2: 4.0 * ureg.nA}
    desired_data = {
        name_all: pd.DataFrame({
                'wavelength (nm)': [1, 2],
                'current (nA)': [3.0, 4.0]})}
    actual_data = exp_units.master_data_dict(data_dict)
    assertDataDictEqual(actual_data, desired_data)

def testGenerateMasterDataDict2Var(exp, exp_data):
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    master_data = pd.DataFrame({
        'wavelength': [0, 1, 2, 0, 1, 2],
        'temperature': [25.0, 25.0, 25.0, 35.0, 35.0, 35.0],
        'Value': [0, 1, 2, 3, 4, 5]})
    names = [
        'TEST1' + js + 'wavelength' + ns + '0' + js + \
            'temperature' + ns + '25.0',
        'TEST1' + js + 'wavelength' + ns + '1' + js + \
            'temperature' + ns + '25.0',
        'TEST1' + js + 'wavelength' + ns + '2' + js + \
            'temperature' + ns + '25.0',
        'TEST1' + js + 'wavelength' + ns + '0' + js + \
            'temperature' + ns + '35.0',
        'TEST1' + js + 'wavelength' + ns + '1' + js + \
            'temperature' + ns + '35.0',
        'TEST1' + js + 'wavelength' + ns + '2' + js + \
            'temperature' + ns + '35.0',
    ]
    data_dict = {
        names[0]: 0,
        names[1]: 1,
        names[2]: 2,
        names[3]: 3,
        names[4]: 4,
        names[5]: 5}
    desired_data = {
        'TEST1' + js + 'wavelength' + ns + 'all':
        {
            'TEST1' + js + 'temperature' + ns + '25.0': pd.DataFrame({
                'wavelength': [0, 1, 2],
                'Value': [0, 1, 2]}),
            'TEST1' + js + 'temperature' + ns + '35.0': pd.DataFrame({
                'wavelength': [0,1,2],
                'Value': [3, 4, 5]})
        },
        'TEST1' + js + 'temperature' + ns + 'all':
        {
            'TEST1' + js + 'wavelength' + ns + '0': pd.DataFrame({
                'temperature': [25.0, 35.0],
                'Value': [0, 3]}),
            'TEST1' + js + 'wavelength' + ns + '1': pd.DataFrame({
                'temperature': [25.0, 35.0],
                'Value': [1, 4] }),
            'TEST1' + js + 'wavelength' + ns + '2': pd.DataFrame({
                'temperature': [25.0, 35.0],
                'Value': [2, 5] })
        },
    }

    actual_data = exp.master_data_dict(data_dict)
    assertDataDictEqual(actual_data, desired_data)

def test_master_data_dict_includue_x(exp, exp_data, convert_name):
    names = [convert_name(name) for name in \
        [
        'TEST1~wavelength=1~temperature=25.0',
        'TEST1~wavelength=2~temperature=25.0',
        'TEST1~wavelength=1~temperature=35.0',
        'TEST1~wavelength=2~temperature=35.0',
        ]]
    name_all = convert_name('TEST1~wavelength=all')
    data_dict = {
        names[0]: 1.0,
        names[1]: 2.0,
        names[2]: 3.0,
        names[3]: 4.0,
    }
    desired_data = {
        convert_name('TEST1~wavelength=all'): {
            convert_name('TEST1~temperature=25.0'):
                pd.DataFrame({
                'wavelength': [1, 2],
                'Value': [1.0, 2.0]}),

            convert_name('TEST1~temperature=35.0'):
                pd.DataFrame({
                'wavelength': [1, 2],
                'Value': [3.0, 4.0]}),
        },
    }
    actual_data = exp.master_data_dict(
            data_dict, x_axis_include=['wavelength'])
    assertDataDictEqual(actual_data, desired_data)

def test_master_data_dict_exclude_x(exp, exp_data, convert_name):
    names = [convert_name(name) for name in \
        [
        'TEST1~wavelength=1~temperature=25.0',
        'TEST1~wavelength=2~temperature=25.0',
        'TEST1~wavelength=1~temperature=35.0',
        'TEST1~wavelength=2~temperature=35.0',
        ]]
    name_all = convert_name('TEST1~temperature=all')
    data_dict = {
        names[0]: 1.0,
        names[1]: 2.0,
        names[2]: 3.0,
        names[3]: 4.0,
    }
    desired_data = {
        convert_name('TEST1~temperature=all'): {
            convert_name('TEST1~wavelength=1'):
                pd.DataFrame({
                'temperature': [25.0, 35.0],
                'Value': [1.0, 3.0]}),

            convert_name('TEST1~wavelength=2'):
                pd.DataFrame({
                'temperature': [25.0, 35.0],
                'Value': [2.0, 4.0]}),
        },
    }
    actual_data = exp.master_data_dict(
            data_dict, x_axis_exclude=['wavelength'])
    assertDataDictEqual(actual_data, desired_data)

def test_master_data_dict_includue_c(exp, exp_data, convert_name):
    names = [convert_name(name) for name in \
        [
        'TEST1~wavelength=1~temperature=25.0',
        'TEST1~wavelength=2~temperature=25.0',
        'TEST1~wavelength=1~temperature=35.0',
        'TEST1~wavelength=2~temperature=35.0',
        ]]
    name_all = convert_name('TEST1~wavelength=all')
    data_dict = {
        names[0]: 1.0,
        names[1]: 2.0,
        names[2]: 3.0,
        names[3]: 4.0,
    }
    desired_data = {
        convert_name('TEST1~wavelength=all'): {
            convert_name('TEST1~temperature=25.0'):
                pd.DataFrame({
                'wavelength': [1, 2],
                'Value': [1.0, 2.0]}),

            convert_name('TEST1~temperature=35.0'):
                pd.DataFrame({
                'wavelength': [1, 2],
                'Value': [3.0, 4.0]}),
        },
    }
    actual_data = exp.master_data_dict(
            data_dict, c_axis_include=['temperature'])
    assertDataDictEqual(actual_data, desired_data)

def test_master_data_dict_exclude_c(exp, exp_data, convert_name):
    names = [convert_name(name) for name in \
        [
        'TEST1~wavelength=1~temperature=25.0',
        'TEST1~wavelength=2~temperature=25.0',
        'TEST1~wavelength=1~temperature=35.0',
        'TEST1~wavelength=2~temperature=35.0',
        ]]
    name_all = convert_name('TEST1~wavelength=all')
    data_dict = {
        names[0]: 1.0,
        names[1]: 2.0,
        names[2]: 3.0,
        names[3]: 4.0,
    }
    desired_data = {
        convert_name('TEST1~wavelength=all'): {
            convert_name('TEST1~temperature=25.0'):
                pd.DataFrame({
                'wavelength': [1, 2],
                'Value': [1.0, 2.0]}),

            convert_name('TEST1~temperature=35.0'):
                pd.DataFrame({
                'wavelength': [1, 2],
                'Value': [3.0, 4.0]}),
        },
    }
    actual_data = exp.master_data_dict(
            data_dict, c_axis_exclude=['wavelength'])
    assertDataDictEqual(actual_data, desired_data)

def testGenerateMasterDataDict3Var(exp, exp_data, convert_name):
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    master_data = pd.DataFrame({
        'wavelength': [0, 0, 0, 0, 1, 1, 1, 1],
        'temperature': [25.0, 25.0, 35.0, 35.0, 25.0, 25.0, 35.0, 35.0],
        'material': ['Au', 'Al', 'Au', 'Al', 'Au', 'Al', 'Au', 'Al'],
        'Value': [0, 1, 2, 3, 4, 5, 6, 7]})
    names = [
        'TEST1' + js + 'wavelength' + ns + '0' + js + \
            'temperature' + ns + '25.0' + js + 'material' + ns + 'Au',
        'TEST1' + js + 'wavelength' + ns + '0' + js + \
            'temperature' + ns + '25.0' + js + 'material' + ns + 'Al',
        'TEST1' + js + 'wavelength' + ns + '0' + js + \
            'temperature' + ns + '35.0' + js + 'material' + ns + 'Au',
        'TEST1' + js + 'wavelength' + ns + '0' + js + \
            'temperature' + ns + '35.0' + js + 'material' + ns + 'Al',
        'TEST1' + js + 'wavelength' + ns + '1' + js + \
            'temperature' + ns + '25.0' + js + 'material' + ns + 'Au',
        'TEST1' + js + 'wavelength' + ns + '1' + js + \
            'temperature' + ns + '25.0' + js + 'material' + ns + 'Al',
        'TEST1' + js + 'wavelength' + ns + '1' + js + \
            'temperature' + ns + '35.0' + js + 'material' + ns + 'Au',
        'TEST1' + js + 'wavelength' + ns + '1' + js + \
            'temperature' + ns + '35.0' + js + 'material' + ns + 'Al',
    ]
    data_dict = {
        names[0]: 0,
        names[1]: 1,
        names[2]: 2,
        names[3]: 3,
        names[4]: 4,
        names[5]: 5,
        names[6]: 6,
        names[7]: 7}
    desired_data = {
        'TEST1' + js + 'wavelength' + ns + 'all' + js + 'material' + \
            ns + 'Au': {
            convert_name('TEST1~temperature-25.0'): pd.DataFrame({
                'wavelength': [0, 1],
                'Value': [0, 4]}),
            convert_name('TEST1~temperature-35.0'): pd.DataFrame({
                'wavelength': [0, 1],
                'Value': [2, 6]}),
        },
        convert_name('TEST1~wavelength-all~material-Al'):
        {
            convert_name('TEST1~temperature-25.0'): pd.DataFrame({
                'wavelength': [0,1],
                'Value': [1,5]}),
            convert_name('TEST1~temperature-35.0'): pd.DataFrame({
                'wavelength': [0,1],
                'Value': [3,7]}),
        },
        convert_name('TEST1~wavelength-all~temperature-25.0'): {
            convert_name('TEST1~material-Au'): pd.DataFrame({
                'wavelength': [0,1],
                'Value': [0,4]}),
            convert_name('TEST1~material-Al'): pd.DataFrame({
                'wavelength': [0,1],
                'Value': [1,5]}),
        },
        convert_name('TEST1~wavelength-all~temperature-35.0'): {
            convert_name('TEST1~material-Au'): pd.DataFrame({
                'wavelength': [0,1],
                'Value': [2,6]}),
            convert_name('TEST1~material-Al'): pd.DataFrame({
                'wavelength': [0,1],
                'Value': [3,7]}),
        },
        convert_name('TEST1~temperature-all~material-Au'): {
            convert_name('TEST1~wavelength-0'): pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [0,2]}),
            convert_name('TEST1~wavelength-1'): pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [4,6]}),
        },
        convert_name('TEST1~temperature-all~material-Al'): {
            convert_name('TEST1~wavelength-0'): pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [1,3]}),
            convert_name('TEST1~wavelength-1'): pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [5,7]}),
        },
        convert_name('TEST1~temperature-all~wavelength-0'): {
            convert_name('TEST1~material-Au'): pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [0,2]}),
            convert_name('TEST1~material-Al'): pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [1,3]}),
        },
        convert_name('TEST1~temperature-all~wavelength-1'): {
            convert_name('TEST1~material-Au'): pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [4,6]}),
            convert_name('TEST1~material-Al'): pd.DataFrame({
                'temperature': [25.0,35.0],
                'Value': [5,7]}),
        },
        convert_name('TEST1~material-all~temperature-25.0'): {
            convert_name('TEST1~wavelength-0'): pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [0,1]}),
            convert_name('TEST1~wavelength-1'): pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [4,5]}),
        },
        convert_name('TEST1~material-all~temperature-35.0'): {
            convert_name('TEST1~wavelength-0'): pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [2,3]}),
            convert_name('TEST1~wavelength-1'): pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [6,7]}),
        },
        convert_name('TEST1~material-all~wavelength-0'): {
            convert_name('TEST1~temperature-25.0'): pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [0,1]}),
            convert_name('TEST1~temperature-35.0'): pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [2,3]}),
        },
        convert_name('TEST1~material-all~wavelength-1'): {
            convert_name('TEST1~temperature-25.0'): pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [4,5]}),
            convert_name('TEST1~temperature-35.0'): pd.DataFrame({
                'material': ['Au', 'Al'],
                'Value': [6,7]}),
        },
    }

    actual_data = exp.master_data_dict(data_dict)
    assertDataDictEqual(actual_data, desired_data)



"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment
from ast import literal_eval
from sciparse import assertDataDictEqual

@pytest.fixture
def exp_data(path_data):
        wavelength = np.array([1, 2])
        temperature = np.array([25, 50])
        replicate = np.array([0, 1])
        frequency = 8500
        fake_data = pd.DataFrame(
            {'Time (ms)': [0, 0.1, 0.2],
             'Voltage (V)': [1, 2, 3]})
        def data_func(cond):
            fake_data = pd.DataFrame(
                {'Time (ms)': [0, 0.1, 0.2],
                 'Voltage (V)': [1, 2, 3]})
            return fake_data
        yield dict({
            'wavelength': wavelength,
            'temperature': temperature,
            'replicate': replicate,
            'frequency': frequency,
            'fake_data': fake_data,
            'data_func': data_func,
            'major_separator': '~',
            'minor_separator': '=',
        }, **path_data)

@pytest.fixture
def exp(exp_data):
    exp = Experiment(
        name='TEST1', kind='test',
         measure_func=exp_data['data_func'],
         frequency=exp_data['frequency'],
         wavelength=exp_data['wavelength'],
         temperature=exp_data['temperature'],
         replicate=exp_data['replicate'])
    yield exp
    rmtree(exp_data['data_base_path'], ignore_errors=True)
    rmtree(exp_data['figures_base_path'], ignore_errors=True)
    rmtree(exp_data['designs_base_path'], ignore_errors=True)

def testExecuteExperimentFilesWritten(exp, exp_data, convert_name):
    """
    Tests that we successfully load a dataset
    """
    files = [
        convert_name('TEST1~wavelength-1~temperature-25~replicate-0'),
        convert_name('TEST1~wavelength-1~temperature-25~replicate-1'),
        convert_name('TEST1~wavelength-1~temperature-50~replicate-0'),
        convert_name('TEST1~wavelength-1~temperature-50~replicate-1'),
        convert_name('TEST1~wavelength-2~temperature-25~replicate-0'),
        convert_name('TEST1~wavelength-2~temperature-25~replicate-1'),
        convert_name('TEST1~wavelength-2~temperature-50~replicate-0'),
        convert_name('TEST1~wavelength-2~temperature-50~replicate-1')]
    full_filenames = [exp_data['data_full_path'] + fn + '.csv' for fn in files]
    exp.Execute()
    files_found = [os.path.isfile(fn) for fn in full_filenames]
    assert_equal(all(files_found), True)

def testExecuteExperimentDataCorrect(exp, exp_data, convert_name):
    exp.Execute()
    desired_data = {
        convert_name('TEST1~wavelength-1~temperature-25~replicate-0'): exp_data['fake_data'],
        convert_name('TEST1~wavelength-1~temperature-25~replicate-1'): exp_data['fake_data'],
        convert_name('TEST1~wavelength-1~temperature-50~replicate-0'):exp_data['fake_data'],
        convert_name('TEST1~wavelength-1~temperature-50~replicate-1'):exp_data['fake_data'],
        convert_name('TEST1~wavelength-2~temperature-25~replicate-0'):exp_data['fake_data'],
        convert_name('TEST1~wavelength-2~temperature-25~replicate-1'):exp_data['fake_data'],
        convert_name('TEST1~wavelength-2~temperature-50~replicate-0'):exp_data['fake_data'],
        convert_name('TEST1~wavelength-2~temperature-50~replicate-1'):exp_data['fake_data'],
    }
    actual_data = exp.data
    assertDataDictEqual(actual_data, desired_data)



"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
from pandas.testing import assert_frame_equal
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment
from sciparse import dict_to_string, assertDataDictEqual
from ast import literal_eval
from itertools import zip_longest
from pathlib import Path

def testExtractConstants(exp, exp_data):
    """
    Tests that our experiment properly extracts metadata from a list of
    arbitrary keyword arguments.
    """
    actual_constants = exp.constants
    desired_constants = {'frequency': exp_data['frequency']}
    assert_equal(actual_constants , desired_constants)

def test_save_raw_results_filename(exp, exp_data):
    """
    Tests that we save the raw results in the proper directory and with
    the proper metadata and with the proper name.
    """
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    raw_data = pd.DataFrame({'wavelength': [1, 2, 3],
                             'Current': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    exp.saveRawResults(raw_data, cond)
    filename_desired = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
                       'temperature' + ns + '25.csv'
    file_found = os.path.isfile(exp_data['data_full_path'] + filename_desired)
    assert_equal(file_found, True)

def test_save_raw_results_metadata(exp, exp_data):
    """
    Tests that we save the raw results in the proper directory and with
    the proper metadata and with the proper name.
    """
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    raw_data = pd.DataFrame({'wavelength': [1, 2, 3],
                             'Current': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency':
            exp_data['frequency']}
    exp.saveRawResults(raw_data, cond)
    filename_desired = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
                       'temperature' + ns + '25.csv'
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        first_line = fh.readline()
        metadata_actual = literal_eval(first_line)

    metadata_desired = {'frequency': exp_data['frequency']}
    assert_equal(metadata_actual, metadata_desired)

def test_save_raw_results_data(exp, exp_data):
    """
    Tests that we correctly save the raw data and can read it out again.
    """
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    data_desired = pd.DataFrame({'wavelength': [1, 2, 3],
                             'Current': [4,4.5,6]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency':
            exp_data['frequency']}
    exp.saveRawResults(data_desired, cond)
    filename_desired = 'TEST1' + js + 'wavelength' + ns + '1' + js + 'temperature' + ns + '25.csv'
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        first_line = fh.readline()
        data_actual = pd.read_csv(fh)
    assert_allclose(data_actual, data_desired)
    assert_equal(data_actual.columns.values, data_desired.columns.values)

def test_save_raw_scalar(exp, exp_data):
    data_to_write = 4.05
    data_desired = pd.DataFrame({
            'wavelength': [1],
            'temperature': [25],
            'Value': [data_to_write]})
    cond = {'wavelength': 1, 'temperature': 25, 'frequency':
            exp_data['frequency']}
    filename_desired = 'TEST1.csv'
    exp.saveRawResults(data_to_write, cond)
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        metadata_actual = literal_eval(fh.readline())
        data_actual = pd.read_csv(fh)
    assert_frame_equal(data_actual, data_desired)

def test_save_raw_scalar_multiple(exp, exp_data):
    data_to_write = 4.05
    data_desired = pd.DataFrame({
            'wavelength': [1, 2],
            'temperature': [25, 25],
            'Value': [data_to_write, data_to_write]})
    cond1 = {'wavelength': 1, 'temperature': 25, 'frequency':
            exp_data['frequency']}
    cond2 = {'wavelength': 2, 'temperature': 25, 'frequency':
            exp_data['frequency']}
    filename_desired = 'TEST1.csv'
    exp.saveRawResults(data_to_write, cond1)
    exp.saveRawResults(data_to_write, cond2)
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        metadata_actual = literal_eval(fh.readline())
        data_actual = pd.read_csv(fh)
    assert_frame_equal(data_actual, data_desired)

def test_save_raw_scalar_multiple_units(ureg, exp_units, exp_data):
    data_to_write = 4.05*ureg.nA
    exp_units.measure_name = 'photocurrent'
    data_desired = pd.DataFrame({
            'wavelength (nm)': np.array([1, 2]),
            'temperature (K)': np.array([25, 25]),
            'photocurrent (nA)': [4.05, 4.05]})
    cond1 = {
        'wavelength': 1*ureg.nm,
        'temperature': 25*ureg.degK,
        'frequency': exp_data['frequency']*ureg.Hz}
    cond2 = {
        'wavelength': 2*ureg.nm,
        'temperature': 25*ureg.degK,
        'frequency': exp_data['frequency']*ureg.Hz}
    filename_desired = 'TEST1.csv'
    exp_units.saveRawResults(data_to_write, cond1)
    exp_units.saveRawResults(data_to_write, cond2)
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        metadata_actual = literal_eval(fh.readline())
        data_actual = pd.read_csv(fh)
    assert_frame_equal(data_actual, data_desired)


@pytest.mark.skip
def testSaveDerivedQuantitiesFilename(exp, exp_data):
    master_data = {'TEST1': pd.DataFrame({'wavelength': [1, 2, 3],
                             'Mean': [1,2,4]})}
    cond = {'wavelength': 1, 'temperature': 25, 'frequency': 8500}
    exp.master_data = master_data
    exp.saveDerivedQuantities()
    filename_desired = 'TEST1.csv'
    file_found = os.path.isfile(exp_data['data_full_path'] + filename_desired)
    assert_equal(file_found, True)

@pytest.mark.skip
def testSaveDerivedQuantitiesData(exp, exp_data):
    data_desired = pd.DataFrame({'wavelength': [1, 2, 3],
                             'Mean': [1,2,4]})
    exp.master_data = data_desired
    exp.saveDerivedQuantities()
    filename_desired = 'TEST1.csv'
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        metadata = fh.readline()
        data_actual = pd.read_csv(fh)
    assert_allclose(data_actual, data_desired)
    assert_equal(data_actual.columns.values, data_desired.columns.values)

@pytest.mark.skip
def testSaveDerivedQuantitiesMetadata(exp, exp_data):
    data_desired = pd.DataFrame({'wavelength': [1, 2, 3],
                             'Mean': [1,2,4]})
    metadata_desired = {'frequency': exp_data['frequency']}
    exp.master_data = data_desired
    exp.saveDerivedQuantities()
    filename_desired = 'TEST1.csv'
    with open(exp_data['data_full_path'] + filename_desired) as fh:
        metadata_actual = literal_eval(fh.readline())
    assert_equal(metadata_actual, metadata_desired)



import pint
import pytest
from xsugar import Experiment
from sciparse import assertDataDictEqual, assert_equal_qt
from pint import UnitRegistry
from numpy.testing import assert_equal
import numpy as np


@pytest.fixture
def exp(ureg):
    wavelength = ureg.nm * np.array([500, 505])
    temperature = ureg.degK * np.array([300, 305])
    my_exp = Experiment(kind='test', name='TEST1')
    return my_exp

def test_units_init(ureg):
    wavelength = ureg.nm * np.array([500, 505])
    temperature = ureg.degK * np.array([25.0, 35.0])
    sampling_frequency = ureg.Hz * 1.0
    my_exp = Experiment(kind='test', name='TEST1',
            wavelength=wavelength, temperature=temperature,
            sampling_frequency=sampling_frequency)
    desired_factors = {
        'wavelength': wavelength,
        'temperature': temperature}
    desired_constants = {'sampling_frequency': sampling_frequency}
    actual_factors = my_exp.factors
    actual_constants = my_exp.constants
    assertDataDictEqual(actual_factors, desired_factors)
    assertDataDictEqual(actual_constants, desired_constants)



"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment, ureg
from sciparse import assertDataDictEqual
from ast import literal_eval

def testNameFromCondition(exp, exp_data):
    """
    Tests whether we can properly generate a name from a specified
    condition
    """
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    filename_desired = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
        'temperature' + ns + '25'
    condition = {'wavelength': 1, 'temperature': 25}
    filename_actual = exp.nameFromCondition(condition)
    assert_equal(filename_actual, filename_desired)

def testNameFromConditionWithID(exp, exp_data):
    """
    Tests whether we can properly generate a name from a specified
    condition
    """
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    filename_desired = 'TEST1' + ns + 'id' + js +  'wavelength' + ns + \
                       '1' + js + 'temperature' + ns + '25'
    condition = {'ident': 'id', 'wavelength': 1, 'temperature': 25}
    filename_actual = exp.nameFromCondition(condition)
    assert_equal(filename_actual, filename_desired)

def test_name_from_condition_units(convert_name):
    sampling_frequency = 1.0 * ureg.Hz
    temperature = ureg.K * np.array([300, 305])
    wavelength = ureg.nm * np.array([500, 505])
    exp = Experiment(name='TEST1', kind='test',
            sampling_frequenc=sampling_frequency,
            wavelength=wavelength, temperature=temperature)
    actual_name = exp.nameFromCondition(exp.conditions[0])
    desired_name = convert_name('TEST1~wavelength=500nm~temperature=300K')
    assert_equal(actual_name, desired_name)

def testConditionFromName(exp, exp_data):
    """
    Tests whether we can generate a condition from a specified name
    """
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    filename_desired = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
                       'temperature' + ns + '25'
    condition_desired = {'wavelength': 1, 'temperature': 25,
                         'frequency': 8500}
    condition_actual = exp.conditionFromName(filename_desired)
    assert_equal(condition_actual, condition_desired)

def testConditionFromNamePartial(exp, exp_data):
    """
    Tests whether we can generate a condition from a specified name
    """
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    filename_desired = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
                       'temperature' + ns + '25'
    condition_desired = {'wavelength': 1, 'temperature': 25}
    condition_actual = exp.conditionFromName(
        filename_desired, full_condition=False)
    assert_equal(condition_actual, condition_desired)

def test_condition_from_name_units(exp,exp_data, convert_name):
    name = convert_name('TEST1~wavelength=100nm~temperature=25K')
    desired_condition = {
        'wavelength': ureg.nm * 100,
        'temperature': ureg.degK * 25,
        'frequency': exp_data['frequency']}
    actual_condition = exp.conditionFromName(name)
    assertDataDictEqual(actual_condition, desired_condition)

def testConditionFromNameMetadata(exp, exp_data):
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    name_desired = 'TEST1' + js + 'wavelength' + ns + '1' + js + \
                       'temperature' + ns + '25'
    condition_desired = {'wavelength': 1, 'temperature': 25,
                         'frequency': 8500,
                         'horn': 'shoe'}
    exp.metadata[name_desired] = \
        {'horn': 'shoe'}

    condition_actual = exp.conditionFromName(name_desired)
    assert_equal(condition_actual, condition_desired)


def testConditionFromNameWithID(exp, exp_data):
    """
    Tests whether we can generate a condition from a specified name
    """
    js, ns = exp_data['major_separator'], exp_data['minor_separator']
    name_desired = 'TEST1' + ns + 'id' + js + 'wavelength' + ns + \
                       '1' + js + 'temperature' + ns + '25'
    condition_desired = {'ident': 'id', 'wavelength': 1,
                         'temperature': 25,
                         'frequency': 8500}
    condition_actual = exp.conditionFromName(name_desired)
    assert_equal(condition_actual, condition_desired)

def test_drop_name(exp, convert_name):
    initial_name = convert_name('TEST1~wavelength=0.01')
    desired_name = convert_name('wavelength=0.01')
    actual_name = exp.drop_name(initial_name)
    assert_equal(actual_name, desired_name)

def test_prettify_name(exp, convert_name):
    initial_name = convert_name('TEST1~wavelength=0.01~temperature=25.0')
    desired_name = 'wavelength=0.01, temperature=25.0'
    actual_name = exp.prettify_name(initial_name)
    assert_equal(actual_name, desired_name)



"""
Tests data reading and writing operation, along with condition generation
"""
import pytest
import numpy as np
import pandas as pd
from pandas.testing import assert_frame_equal
import os
from shutil import rmtree
from numpy.testing import assert_equal, assert_allclose
from xsugar import Experiment
from sciparse import assertDataDictEqual
from ast import literal_eval
from itertools import zip_longest
from pathlib import Path

def testLoadData(exp, exp_data, convert_name):
    data_desired = pd.DataFrame({'wavelengths': [1, 2, 3],
                             'Mean': [1,2,4]})
    metadata_desired = {'non': 'sense'}
    filename = convert_name('TEST1~wavelengths-1~temperatures-25')
    file_extension = '.csv'
    full_filename = exp_data['data_full_path'] + filename + file_extension
    with open(full_filename, 'w+') as fh:
        fh.write(str(metadata_desired) + '\n')
        data_desired.to_csv(fh, mode='a', index=False)
    exp.loadData()
    data_actual = exp.data[filename]
    assert_frame_equal(data_actual, data_desired)

def testLoadXRDData(convert_name):
    file_location = os.path.dirname(os.path.abspath(__file__))
    base_path = file_location + '/data'
    exp = Experiment(name='TEST1', kind='test', base_path=base_path)
    exp.loadXRDData()
    data_desired = pd.DataFrame({
        'Angle (deg)': [69.05, 69.055, 69.06, 69.065, 69.07,69.075,69.08,
        69.085, 69.09, 69.095, 69.1, 69.105, 69.11, 69.115],
        'Counts': [24, 30, 28, 40, 132, 272, 3472, 16368,21970,10562,
                   1210,264,130,64]})
    data_actual = exp.data[convert_name('TEST1~1~type-locked_coupled~peak-Si')]
    assert_frame_equal(data_actual, data_desired)

def testLoadXRDMetadata(convert_name):
    file_location = os.path.dirname(os.path.abspath(__file__))
    base_path = file_location + '/data'
    exp = Experiment(name='TEST1', kind='test', base_path=base_path)
    exp.loadXRDData()
    metadata_desired = {
        'date': '02/10/2021',
        'increment': 0.005, 'scantype': 'locked coupled',
        'start': 69.05, 'steps': 14, 'time': 1,
        'theta': 34.0, '2theta': 68.0, 'phi': 180.13, 'chi': -0.972}
    metadata_actual = exp.metadata[convert_name('TEST1~1~type-locked_coupled~peak-Si')]
    assert_equal(metadata_actual, metadata_desired)

def testLoadConstants(exp, exp_data, convert_name):
    """
    Tests that we can load metadata from a file successfully
    """
    wavelengths = np.array([1, 2, 3])
    temperatures = np.array([25, 50])
    frequency = 8500
    with open(exp_data['data_full_path'] + \
              convert_name('TEST1~wavelengths-1~temperatures-25.csv'), 'w+') as fh:
        fh.write('{"frequency": 8500}\n')
        fh.write(f'Time, Data\n')
        fh.write(f'1, 2\n')
    exp = Experiment(name='TEST1', kind='test')
    exp.loadData()
    constants_actual = exp.constants
    constants_desired = {'frequency': frequency}
    assert_equal(constants_actual, constants_desired)

def testLoadMetadata(exp, exp_data, convert_name):
    """
    Tests that we can load metadata from a file successfully
    """
    wavelengths = np.array([1, 2, 3])
    temperatures = np.array([25, 50])
    frequency = 8500
    with open(exp_data['data_full_path'] + \
              convert_name('TEST1~wavelengths-1~temperatures-25.csv'), 'w+') as fh:
        fh.write('{"frequency": 8500}\n')
        fh.write(f'Time, Data\n')
        fh.write(f'1, 2\n')
    exp = Experiment(name='TEST1', kind='test')
    exp.loadData()
    metadata_actual = exp.metadata
    metadata_desired = {
        convert_name('TEST1~wavelengths-1~temperatures-25'): \
                        {'frequency': frequency}}
    assert_equal(metadata_actual, metadata_desired)

# TODO: ADD TEST CASE TO ENSURE WE DON'T LOAD IN TOO MUCH DATA, OR DATA
# THAT DOES NOT PRECISELY MATCH *BOTH* THE NAME *AND* THE ID.

# TODO: ADD TEST CASE TO ENSURE WE DON'T LOAD IN TOO MUCH DATA, OR DATA
# THAT DOES NOT PRECISELY MATCH *BOTH* THE NAME *AND* THE ID.

def testLookup(exp, convert_name):
    fudge_data = pd.DataFrame(
        {'Time (ms)': [1, 2, 3],
        'Voltage (V)': [0,0.1, 0.2]})
    exp.data = {
        convert_name('TEST1~wavelengths-1~temperatures-25'):fudge_data,
        convert_name('TEST1~wavelengths-2~temperatures-25'):fudge_data,
        convert_name('TEST1~wavelengths-2~temperatures-35'):fudge_data,
        convert_name('TEST1~wavelengths-2~temperatures-35'):fudge_data,
                    }
    data_actual = exp.lookup(temperatures=25)
    data_desired = {
        convert_name('TEST1~wavelengths-1~temperatures-25'):fudge_data,
        convert_name('TEST1~wavelengths-2~temperatures-25'):fudge_data,}
    assertDataDictEqual(data_actual, data_desired)



import pytest
from xsugar import ureg, dc_photocurrent, modulated_photocurrent, noise_current, Experiment
from sciparse import assert_equal_qt, assert_allclose_qt, assertDataDictEqual
from sugarplot import assert_figures_equal, plt, Figure
import pandas as pd
import numpy as np
import os

@pytest.fixture
def sim_exp():
    file_location = os.path.dirname(os.path.abspath(__file__))
    base_path = file_location + '/data'
    sim_exp = Experiment(name='REFL2', kind='test', base_path=base_path)
    return sim_exp

def test_dc_photocurrent():
    gain = 10 * ureg.Mohm
    input_data = pd.DataFrame({
            'time (ms)': [0, 0.01, 0.02],
            'voltage (uV)': [0.1, 0.2, 0.3]})
    desired_photocurrent = 2e-5*ureg.nA
    cond = {'gain': gain}
    actual_photocurrent = dc_photocurrent(input_data, cond)
    assert_allclose_qt(actual_photocurrent, desired_photocurrent)

def test_modulated_photocurrent():
    data_length = 1000
    sampling_frequency = 1 * ureg.kHz
    signal_frequency = 0.1 * ureg.kHz
    sampling_period = (1 / sampling_frequency).to(ureg.s)
    gain = 1.5 * ureg.Mohm
    cond = {'gain': gain, 'sampling_frequency': 1 * ureg.kHz}

    number_periods = int(np.floor(data_length / (sampling_frequency / signal_frequency)))
    number_sync_points = number_periods
    indices = np.arange(0, number_sync_points, 1)
    sync_indices = (1/2* sampling_frequency / signal_frequency *(1 + 2*indices)).astype(np.int).magnitude
    zero_column = np.zeros(data_length, dtype=np.int)
    zero_column[sync_indices] = 1

    times = np.arange(0, data_length, 1)*sampling_period
    voltages = 0.5 * ureg.mV * np.sin(2*np.pi * signal_frequency * times)
    data = pd.DataFrame({
            'time (ms)': times.to(ureg.ms).magnitude,
            'voltage (mV)': voltages.to(ureg.mV).magnitude,
            'Sync': zero_column
            })
    desired_data = (0.5 * ureg.mV / gain).to(ureg.pA) / np.sqrt(2)
    actual_data = modulated_photocurrent(data, cond)
    assert_allclose_qt(actual_data, desired_data)

def test_noise_current():
    data_length = 10000
    sampling_frequency = 10*ureg.kHz
    nyquist_frequency = sampling_frequency / 2
    sampling_period = (1 / sampling_frequency).to(ureg.s)
    noise_voltage = np.random.normal(size=data_length)*ureg.uV
    filter_cutoff = 200*ureg.Hz
    times = np.arange(0, data_length, 1)*sampling_period
    data = pd.DataFrame({
            'Time (ms)': times.to(ureg.ms).magnitude,
            'voltage (uV)': noise_voltage.to(ureg.uV).magnitude,
            })
    gain = 1.0 * ureg.Mohm
    cond = {
        'gain': gain,
        'filter_cutoff': filter_cutoff
    }
    filter_correction_factor = (nyquist_frequency - filter_cutoff) / \
                               nyquist_frequency
    desired_noise_current_psd = \
         np.square(noise_voltage.std() / gain) / nyquist_frequency
    desired_noise_current_psd = desired_noise_current_psd.to(
            ureg.A ** 2 / ureg.Hz)
    actual_noise_current_psd = noise_current(data, cond)
    assert_allclose_qt(
            actual_noise_current_psd, desired_noise_current_psd,
            atol=1e-31, rtol=1e-2)

def test_noise_current_bin2():
    data_length = 3000
    sampling_frequency = 10*ureg.kHz
    sampling_period = (1 / sampling_frequency).to(ureg.s)
    noise_voltage = np.random.normal(size=data_length)*ureg.uV
    times = np.arange(0, data_length, 1)*sampling_period
    data = pd.DataFrame({
            'Time (ms)': times.to(ureg.ms).magnitude,
            'voltage (uV)': noise_voltage.to(ureg.uV).magnitude,
            })
    gain = 1.0 * ureg.Mohm
    cond = {
        'gain': gain,
        'filter_cutoff': 200*ureg.Hz
    }
    desired_noise_current_psd = \
         np.square(noise_voltage.std() / gain) / (sampling_frequency / 2)
    desired_noise_current_psd = desired_noise_current_psd.to(
            ureg.A ** 2 / ureg.Hz)
    actual_noise_current_psd = noise_current(data, cond)
    assert_allclose_qt(
            actual_noise_current_psd, desired_noise_current_psd,
            atol=1e-31, rtol=1e-2)

def test_process_photocurrent_simple(convert_name, sim_exp):
    """
    Verifies that, given a sinusoidal input with a known offset and amplitude, the correct data is generated.
    """
    wavelength = np.array([700, 750]) * ureg.nm
    material = ['Au', 'Al']
    gain = 1 * ureg.Mohm
    current_offset = 100
    current_amplitude = 1
    dR_R0_ratio = current_amplitude / current_offset

    reference_condition = dict(material='Au')
    sin_data = pd.DataFrame({
            'Time (ms)': np.array([0, 1, 2, 3, 4]),
            'Voltage (mV)': current_offset + \
                current_amplitude * np.array([0, -1, 0, 1, 0]),
            'Sync': np.array([1, 0, 0, 0, 1]),
            })
    test_data = {
        convert_name('TEST1~wavelength=700nm~material=Au'): sin_data,
        convert_name('TEST1~wavelength=750nm~material=Au'): sin_data,
        convert_name('TEST1~wavelength=700nm~material=Al'): sin_data,
        convert_name('TEST1~wavelength=750nm~material=Al'): sin_data,
    }
    exp = Experiment(
            name='TEST1', kind='test',
            wavelength=wavelength, material=material, gain=gain)
    exp.data = test_data

    R0_actual, dR_actual, inoise_actual = exp.process_photocurrent(
            reference_condition=reference_condition, sim_exp=sim_exp)
    R0_desired = {
        convert_name('TEST1~wavelength=700nm~material=Au'): 0.93329,
        convert_name('TEST1~wavelength=750nm~material=Au'): 0.948615,
        convert_name('TEST1~wavelength=700nm~material=Al'): 0.93329,
        convert_name('TEST1~wavelength=750nm~material=Al'): 0.948615,
    }
    dR_desired = {
        convert_name('TEST1~wavelength=700nm~material=Au'): \
            0.93329 / np.sqrt(2) * dR_R0_ratio,
        convert_name('TEST1~wavelength=750nm~material=Au'): \
            0.948615 / np.sqrt(2) * dR_R0_ratio,
        convert_name('TEST1~wavelength=700nm~material=Al'): \
            0.93329 / np.sqrt(2) * dR_R0_ratio,
        convert_name('TEST1~wavelength=750nm~material=Al'): \
            0.948615 / np.sqrt(2) * dR_R0_ratio,
    }
    inoise_desired = {
        convert_name('TEST1~wavelength=700nm~material=Au'): \
            8.000000000000231e-22 * ureg.A ** 2 / ureg.Hz,
        convert_name('TEST1~wavelength=750nm~material=Au'): \
            8.000000000000231e-22 * ureg.A ** 2 / ureg.Hz,
        convert_name('TEST1~wavelength=700nm~material=Al'): \
            8.000000000000231e-22 * ureg.A ** 2 / ureg.Hz,
        convert_name('TEST1~wavelength=750nm~material=Al'): \
            8.000000000000231e-22 * ureg.A ** 2 / ureg.Hz,
    }
    assertDataDictEqual(R0_actual, R0_desired)
    assertDataDictEqual(dR_actual, dR_desired)
    assertDataDictEqual(inoise_actual, inoise_desired)

def test_plot_photocurrent_simple(convert_name, sim_exp):
    """
    Verifies that, given a sinusoidal input with a known offset and amplitude, the correct data is generated.
    """
    wavelength = np.array([700, 750]) * ureg.nm
    material = ['Au', 'Al']
    gain = 1 * ureg.Mohm
    current_offset = 100
    current_amplitude = 1
    dR_R0_ratio = current_amplitude / current_offset

    reference_condition = dict(material='Au')
    sin_data = pd.DataFrame({
            'Time (ms)': np.array([0, 1, 2, 3, 4]),
            'Voltage (mV)': current_offset + \
                current_amplitude * np.array([0, -1, 0, 1, 0]),
            'Sync': np.array([1, 0, 0, 0, 1]),
            })
    test_data = {
        convert_name('TEST1~wavelength=700nm~material=Au'): sin_data,
        convert_name('TEST1~wavelength=750nm~material=Au'): sin_data,
        convert_name('TEST1~wavelength=700nm~material=Al'): sin_data,
        convert_name('TEST1~wavelength=750nm~material=Al'): sin_data,
    }
    exp = Experiment(
            name='TEST1', kind='test',
            wavelength=wavelength, material=material, gain=gain)
    exp.data = test_data

    R0_desired = {
        convert_name('TEST1~wavelength=700nm~material=Au'): 0.93329,
        convert_name('TEST1~wavelength=750nm~material=Au'): 0.948615,
        convert_name('TEST1~wavelength=700nm~material=Al'): 0.93329,
        convert_name('TEST1~wavelength=750nm~material=Al'): 0.948615,
    }
    dR_desired = {
        convert_name('TEST1~wavelength=700nm~material=Au'): \
            0.93329 / np.sqrt(2) * dR_R0_ratio,
        convert_name('TEST1~wavelength=750nm~material=Au'): \
            0.948615 / np.sqrt(2) * dR_R0_ratio,
        convert_name('TEST1~wavelength=700nm~material=Al'): \
            0.93329 / np.sqrt(2) * dR_R0_ratio,
        convert_name('TEST1~wavelength=750nm~material=Al'): \
            0.948615 / np.sqrt(2) * dR_R0_ratio,
    }
    inoise_desired = {
        convert_name('TEST1~wavelength=700nm~material=Au'): \
            8.000000000000231e-22 * ureg.A ** 2 / ureg.Hz,
        convert_name('TEST1~wavelength=750nm~material=Au'): \
            8.000000000000231e-22 * ureg.A ** 2 / ureg.Hz,
        convert_name('TEST1~wavelength=700nm~material=Al'): \
            8.000000000000231e-22 * ureg.A ** 2 / ureg.Hz,
        convert_name('TEST1~wavelength=750nm~material=Al'): \
            8.000000000000231e-22 * ureg.A ** 2 / ureg.Hz,
    }
    (R0_figs_actual, _, dR_figs_actual, _, inoise_figs_actual, _) =  \
         exp.plot_photocurrent(
                 reference_condition=reference_condition, sim_exp=sim_exp)

    R0_fig_desired = Figure()
    R0_ax = R0_fig_desired.subplots()
    R0_ax.plot([700, 750], [0.93329, 0.948615])
    R0_ax.plot([700, 750], [0.93329, 0.948615])
    R0_ax.set_xlabel('wavelength (nm)')
    R0_ax.set_ylabel(r'$R_0$')

    dR_fig_desired = Figure()
    dR_ax = dR_fig_desired.subplots()
    dR_ax.plot([700, 750],
            [0.93329 / np.sqrt(2) * dR_R0_ratio,
            0.948615 / np.sqrt(2) * dR_R0_ratio])
    dR_ax.plot([700, 750],
            [0.93329 / np.sqrt(2) * dR_R0_ratio,
            0.948615 / np.sqrt(2) * dR_R0_ratio])
    dR_ax.set_xlabel('wavelength (nm)')
    dR_ax.set_ylabel(r'$\Delta R_{rms}$')

    inoise_fig_desired = Figure()
    inoise_ax = inoise_fig_desired.subplots()
    inoise_ax.plot([700, 750],
            10*np.log10(np.array([8.000000000000231e-22, 8.000000000000231e-22])))
    inoise_ax.plot([700, 750],
            10*np.log10(np.array([8.000000000000231e-22, 8.000000000000231e-22])))
    inoise_ax.set_xlabel('wavelength (nm)')
    inoise_ax.set_ylabel('Power (dBA/Hz)')

    assert_figures_equal(R0_figs_actual[0], R0_fig_desired, atol=1e-6)
    assert_figures_equal(dR_figs_actual[0], dR_fig_desired, atol=1e-6)
    assert_figures_equal(inoise_figs_actual[0], inoise_fig_desired, atol=1e-6)



import setuptools
import os
import glob

example_data = glob.glob('xsugar/examples/*')
source_data = glob.glob('xsugar/source/*')
test_data = glob.glob('xsugar/test/*')
test_data_data1 = glob.glob('xsugar/test/data/data/test/TEST1/*')
test_data_data2 = glob.glob('xsugar/test/data/data/test/REFL2/*')
total_data = example_data + source_data + test_data + test_data_data1 + \
    test_data_data2
package_data = [x.replace('xsugar/', '') for x in total_data]

#raise ValueError(f'PACKAGE DATA: {package_data}')

with open("README.md", "r") as fh:
	long_description = fh.read()

	setuptools.setup(
		name="xsugar",
		version="0.1." + str(os.environ['GITHUB_RUN_NUMBER']),
		author="Jordan Edmunds",
		author_email="edmundsj@uci.edu",
		description="Experimental design, data analysis, and plotting framework in python",
		long_description=long_description,
		long_description_content_type="text/markdown",
		url="https://github.com/edmundsj/xsugar",
		packages=setuptools.find_packages(),
		include_package_data=True,
		package_data={
			"xsugar": package_data
			},
		classifiers=[
			"Programming Language :: Python :: 3",
			"License :: OSI Approved :: MIT License",
			"Operating System :: OS Independent",
		],
		python_requires='>=3.6',
		install_requires=[
				'numpy>=1.14.5',
				'matplotlib>=2.0.0',
				'pandas>=0.24.0',
				'scipy>=1.2.2',
				'pyyaml>=5.1.1',
                'pint>=0.16.0',
                'sugarplot',
                'spectralpy',
                'sciparse',
                'liapy',
		],
	license="MIT",
	)
